# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

PATTERN = r"(BADGIE\s+TIME|BADGIE\s+ME)"

PATTERN_START = r"<!--\s+" + PATTERN + r"\s+-->"

PATTERN_END = r"<!--\s+END\s+" + PATTERN + r"\s+-->"



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

from typing import Optional

from .. import tokens as to
from ..models import Context, ProjectRemote, Remote, RemoteMatch
from ..project import get_project_remotes

REMOTES = {
    RemoteMatch(host="github.com"): {to.GITHUB},
    RemoteMatch(host="gitlab.com"): {to.GITLAB},
    RemoteMatch(host="gitlab.com", path_prefix="saferatday0/library"): {
        to.SAFERATDAY0_LIBRARY
    },
}


def match_remote(project_remote: ProjectRemote) -> Optional[Remote]:
    nodes = []
    for match, tokens in REMOTES.items():
        if project_remote.host != match.host:
            continue
        if match.path_prefix and not project_remote.path.startswith(match.path_prefix):
            continue
        nodes.append(
            Remote(tokens=tokens, host=project_remote.host, path=project_remote.path)
        )
    return nodes


def run(_context: Context) -> list[Remote]:
    remote = get_project_remotes()["origin"]["fetch"]
    return match_remote(remote)



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

from .. import tokens as to
from ..models import Badge
from ._base import register_badges

register_badges(
    {
        to.PRE_COMMIT_CONFIG: Badge(
            name="pre-commit-enabled",
            description="Show that the repository is using pre-commit.",
            example="https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit",
            title="pre-commit",
            link="https://github.com/pre-commit/pre-commit",
            image="https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit",
            weight=10,
        )
    }
)



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

from .. import tokens as to
from ..models import Badge
from ._base import register_badges

register_badges(
    {
        to.SAFERATDAY0_LIBRARY: Badge(
            name=f"saferatday0-library",
            description=f"This component is part of the saferatday0 library.",
            example=f"https://img.shields.io/badge/saferatday0-library-009670?labelColor=303030",
            title=f"saferatday0 library",
            link="https://saferatday0.dev",
            image=f"https://img.shields.io/badge/saferatday0-library-009670?labelColor=303030",
            weight=-100,
        ),
    }
)



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

from .. import tokens as to
from ..models import Badge
from ._base import register_badges

register_badges(
    {
        to.PYTHON_BANDIT: Badge(
            name="python-bandit",
            description="This project uses [Bandit](https://github.com/PyCQA/bandit).",
            example="https://img.shields.io/badge/security-bandit-yellow.svg",
            title="security: bandit",
            link="https://github.com/PyCQA/bandit",
            image="https://img.shields.io/badge/security-bandit-yellow.svg",
            weight=20,
        ),
        to.PYTHON_BLACK: Badge(
            name="python-black",
            description="This project uses [Black](https://github.com/psf/black).",
            example="https://img.shields.io/badge/code_style-black-000000.svg",
            title="code style: black",
            link="https://github.com/psf/black",
            image="https://img.shields.io/badge/code_style-black-000000.svg",
            weight=20,
        ),
        to.PYTHON_DOCFORMATTER: Badge(
            name="python-docformatter",
            description="This project uses [docformatter](https://github.com/PyCQA/docformatter).",
            example="https://img.shields.io/badge/formatter-docformatter-fedcba.svg",
            title="formatter: docformatter",
            link="https://github.com/PyCQA/docformatter",
            image="https://img.shields.io/badge/formatter-docformatter-fedcba.svg",
            weight=20,
        ),
        to.PYTHON_ISORT: Badge(
            name="python-isort",
            description="This project uses [isort](https://github.com/PyCQA/isort).",
            example="https://img.shields.io/badge/imports-isort-1674b1?style=flat&labelColor=ef8336",
            title="imports: isort",
            link="https://pycqa.github.io/isort/",
            image="https://img.shields.io/badge/imports-isort-1674b1?style=flat&labelColor=ef8336",
            weight=20,
        ),
        to.PYTHON_MYPY: Badge(
            name="python-mypy",
            description="This project uses [mypy](https://github.com/python/mypy).",
            example="https://img.shields.io/badge/mypy-checked-2a6db2",
            title="Checked with mypy",
            link="https://mypy-lang.org/",
            image="https://img.shields.io/badge/mypy-checked-2a6db2",
            weight=20,
        ),
    }
)



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

from .. import tokens as to
from ..models import Badge
from ._base import register_badges

register_badges(
    {
        to.CICI_TOOLS: Badge(
            name="cici",
            description="This project uses [cici](https://gitlab.com/saferatday0/cici).",
            example="https://img.shields.io/badge/%E2%9A%A1_cici-enabled-c0ff33",
            title="cici enabled",
            link="https://gitlab.com/saferatday0/cici",
            image="https://img.shields.io/badge/%E2%9A%A1_cici-enabled-c0ff33",
            weight=20,
        ),
    }
)



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

import pytest

from badgie.project import (
    ProjectRemote,
    get_match_group,
    get_project_remotes_from_text,
    match_remote_url,
)


@pytest.mark.parametrize(
    "url,scheme,user,host,path",
    (
        (
            "https://gitlab.com/saferatday0/ansible/roles/kubectl.git",
            "https",
            None,
            "gitlab.com",
            "saferatday0/ansible/roles/kubectl",
        ),
        (
            "git@gitlab.com:saferatday0/ansible/roles/kubectl.git",
            None,
            "git",
            "gitlab.com",
            "saferatday0/ansible/roles/kubectl",
        ),
    ),
)
def test_get_project_remote(url, scheme, user, host, path):
    match = match_remote_url(url)
    assert get_match_group(match, "scheme") == scheme
    assert get_match_group(match, "user") == user
    assert match.group("host") == host
    assert match.group("path") == path


def test_get_project_remotes_from_text():
    text = (
        "origin	git@gitlab.com:saferatday0/badgie.git (fetch)\n"
        "origin	git@gitlab.com:saferatday0/badgie.git (push)\n"
    )
    remote = ProjectRemote(
        name="origin",
        type="fetch",
        url="git@gitlab.com:saferatday0/badgie.git",
        user="git",
        host="gitlab.com",
        path="saferatday0/badgie",
        scheme=None,
    )
    remotes = get_project_remotes_from_text(text)
    assert remotes[remote.name][remote.type].user == remote.user
    assert remotes[remote.name][remote.type].host == remote.host
    assert remotes[remote.name][remote.type].url == remote.url
    assert remotes[remote.name][remote.type].path == remote.path
    assert remotes[remote.name][remote.type].name == remote.name
    assert remotes[remote.name][remote.type].type == remote.type
    assert remotes[remote.name][remote.type].scheme == remote.scheme
